# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
from ....event.model.event import *
import attr




@to_json_decorator
@attr.s
class Sender(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    id_type = attr.ib(type=str, default=None, metadata={'json': 'id_type'})
    sender_type = attr.ib(type=str, default=None, metadata={'json': 'sender_type'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class MentionEvent(object):
    key = attr.ib(type=str, default=None, metadata={'json': 'key'})
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class EventMessage(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["create_time"])
    message_id = attr.ib(type=str, default=None, metadata={'json': 'message_id'})
    root_id = attr.ib(type=str, default=None, metadata={'json': 'root_id'})
    parent_id = attr.ib(type=str, default=None, metadata={'json': 'parent_id'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    chat_type = attr.ib(type=str, default=None, metadata={'json': 'chat_type'})
    message_type = attr.ib(type=str, default=None, metadata={'json': 'message_type'})
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    mentions = attr.ib(type=List[MentionEvent], default=None, metadata={'json': 'mentions'})


@to_json_decorator
@attr.s
class Mention(object):
    key = attr.ib(type=str, default=None, metadata={'json': 'key'})
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    id_type = attr.ib(type=str, default=None, metadata={'json': 'id_type'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class MessageBody(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})


@to_json_decorator
@attr.s
class Message(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["create_time", "update_time"])
    message_id = attr.ib(type=str, default=None, metadata={'json': 'message_id'})
    root_id = attr.ib(type=str, default=None, metadata={'json': 'root_id'})
    parent_id = attr.ib(type=str, default=None, metadata={'json': 'parent_id'})
    msg_type = attr.ib(type=str, default=None, metadata={'json': 'msg_type'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    update_time = attr.ib(type=int, default=None, metadata={'json': 'update_time'})
    deleted = attr.ib(type=bool, default=None, metadata={'json': 'deleted'})
    updated = attr.ib(type=bool, default=None, metadata={'json': 'updated'})
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    sender = attr.ib(type=Sender, default=None, metadata={'json': 'sender'})
    body = attr.ib(type=MessageBody, default=None, metadata={'json': 'body'})
    mentions = attr.ib(type=List[Mention], default=None, metadata={'json': 'mentions'})
    upper_message_id = attr.ib(type=str, default=None, metadata={'json': 'upper_message_id'})


@to_json_decorator
@attr.s
class I18nNames(object):
    zh_cn = attr.ib(type=str, default=None, metadata={'json': 'zh_cn'})
    en_us = attr.ib(type=str, default=None, metadata={'json': 'en_us'})
    ja_jp = attr.ib(type=str, default=None, metadata={'json': 'ja_jp'})


@to_json_decorator
@attr.s
class Chat(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    only_owner_add = attr.ib(type=bool, default=None, metadata={'json': 'only_owner_add'})
    share_allowed = attr.ib(type=bool, default=None, metadata={'json': 'share_allowed'})
    only_owner_at_all = attr.ib(type=bool, default=None, metadata={'json': 'only_owner_at_all'})
    only_owner_edit = attr.ib(type=bool, default=None, metadata={'json': 'only_owner_edit'})
    owner_user_id = attr.ib(type=str, default=None, metadata={'json': 'owner_user_id'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    


@to_json_decorator
@attr.s
class Emoji(object):
    emoji_type = attr.ib(type=str, default=None, metadata={'json': 'emoji_type'})


@to_json_decorator
@attr.s
class Reaction(object):
    reaction_type = attr.ib(type=Emoji, default=None, metadata={'json': 'reaction_type'})
    operator_id = attr.ib(type=str, default=None, metadata={'json': 'operator_id'})
    action_time = attr.ib(type=str, default=None, metadata={'json': 'action_time'})


@to_json_decorator
@attr.s
class MessageReaction(object):
    reaction_id = attr.ib(type=str, default=None, metadata={'json': 'reaction_id'})
    operator_id = attr.ib(type=str, default=None, metadata={'json': 'operator_id'})
    action_time = attr.ib(type=str, default=None, metadata={'json': 'action_time'})
    reaction_type = attr.ib(type=Emoji, default=None, metadata={'json': 'reaction_type'})


@to_json_decorator
@attr.s
class UserId(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})
    union_id = attr.ib(type=str, default=None, metadata={'json': 'union_id'})


@to_json_decorator
@attr.s
class ListEventModerator(object):
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})
    user_id = attr.ib(type=UserId, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class ModeratorList(object):
    added_member_list = attr.ib(type=List[ListEventModerator], default=None, metadata={'json': 'added_member_list'})
    removed_member_list = attr.ib(type=List[ListEventModerator], default=None, metadata={'json': 'removed_member_list'})


@to_json_decorator
@attr.s
class EventSender(object):
    sender_id = attr.ib(type=UserId, default=None, metadata={'json': 'sender_id'})
    sender_type = attr.ib(type=str, default=None, metadata={'json': 'sender_type'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class EventMessageReader(object):
    reader_id = attr.ib(type=UserId, default=None, metadata={'json': 'reader_id'})
    read_time = attr.ib(type=str, default=None, metadata={'json': 'read_time'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class ChatChange(object):
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    add_member_permission = attr.ib(type=str, default=None, metadata={'json': 'add_member_permission'})
    share_card_permission = attr.ib(type=str, default=None, metadata={'json': 'share_card_permission'})
    at_all_permission = attr.ib(type=str, default=None, metadata={'json': 'at_all_permission'})
    edit_permission = attr.ib(type=str, default=None, metadata={'json': 'edit_permission'})
    membership_approval = attr.ib(type=str, default=None, metadata={'json': 'membership_approval'})
    join_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'join_message_visibility'})
    leave_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'leave_message_visibility'})
    moderation_permission = attr.ib(type=str, default=None, metadata={'json': 'moderation_permission'})
    owner_id = attr.ib(type=UserId, default=None, metadata={'json': 'owner_id'})
    


@to_json_decorator
@attr.s
class ChatMemberUser(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})
    user_id = attr.ib(type=UserId, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class ChatAnnouncement(object):
    pass


@to_json_decorator
@attr.s
class ChatMember(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class ChatMemberBot(object):
    bot_id = attr.ib(type=str, default=None, metadata={'json': 'bot_id'})


@to_json_decorator
@attr.s
class ChatMembers(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class File(object):
    pass


@to_json_decorator
@attr.s
class Image(object):
    pass


@to_json_decorator
@attr.s
class ListMember(object):
    member_id_type = attr.ib(type=str, default=None, metadata={'json': 'member_id_type'})
    member_id = attr.ib(type=str, default=None, metadata={'json': 'member_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class ListModerator(object):
    user_id_type = attr.ib(type=str, default=None, metadata={'json': 'user_id_type'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})


@to_json_decorator
@attr.s
class MessageResource(object):
    pass


@to_json_decorator
@attr.s
class UrgentReceivers(object):
    user_id_list = attr.ib(type=List[str], default=None, metadata={'json': 'user_id_list'})


@to_json_decorator
@attr.s
class ListChat(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    owner_id_type = attr.ib(type=str, default=None, metadata={'json': 'owner_id_type'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})
    


@to_json_decorator
@attr.s
class ReadUser(object):
    user_id_type = attr.ib(type=str, default=None, metadata={'json': 'user_id_type'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    timestamp = attr.ib(type=str, default=None, metadata={'json': 'timestamp'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})




@attr.s
class MessageListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[Message], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class MessagePatchReqBody(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})



@to_json_decorator
@attr.s
class MessageReplyReqBody(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    msg_type = attr.ib(type=str, default=None, metadata={'json': 'msg_type'})



@to_json_decorator
@attr.s
class MessageCreateReqBody(object):
    receive_id = attr.ib(type=str, default=None, metadata={'json': 'receive_id'})
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    msg_type = attr.ib(type=str, default=None, metadata={'json': 'msg_type'})






@attr.s
class MessageReadUsersResult(object):
    items = attr.ib(type=List[ReadUser], default=None, metadata={'json': 'items'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})


@to_json_decorator
@attr.s
class ChatUpdateReqBody(object):
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    add_member_permission = attr.ib(type=str, default=None, metadata={'json': 'add_member_permission'})
    share_card_permission = attr.ib(type=str, default=None, metadata={'json': 'share_card_permission'})
    at_all_permission = attr.ib(type=str, default=None, metadata={'json': 'at_all_permission'})
    edit_permission = attr.ib(type=str, default=None, metadata={'json': 'edit_permission'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    join_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'join_message_visibility'})
    leave_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'leave_message_visibility'})
    membership_approval = attr.ib(type=str, default=None, metadata={'json': 'membership_approval'})
    




@attr.s
class FileCreateResult(object):
    file_key = attr.ib(type=str, default=None, metadata={'json': 'file_key'})





@attr.s
class ChatListResult(object):
    items = attr.ib(type=List[ListChat], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})



@attr.s
class ImageCreateResult(object):
    image_key = attr.ib(type=str, default=None, metadata={'json': 'image_key'})







@attr.s
class ChatGetResult(object):
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    add_member_permission = attr.ib(type=str, default=None, metadata={'json': 'add_member_permission'})
    share_card_permission = attr.ib(type=str, default=None, metadata={'json': 'share_card_permission'})
    at_all_permission = attr.ib(type=str, default=None, metadata={'json': 'at_all_permission'})
    edit_permission = attr.ib(type=str, default=None, metadata={'json': 'edit_permission'})
    owner_id_type = attr.ib(type=str, default=None, metadata={'json': 'owner_id_type'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    chat_mode = attr.ib(type=str, default=None, metadata={'json': 'chat_mode'})
    chat_type = attr.ib(type=str, default=None, metadata={'json': 'chat_type'})
    chat_tag = attr.ib(type=str, default=None, metadata={'json': 'chat_tag'})
    join_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'join_message_visibility'})
    leave_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'leave_message_visibility'})
    membership_approval = attr.ib(type=str, default=None, metadata={'json': 'membership_approval'})
    moderation_permission = attr.ib(type=str, default=None, metadata={'json': 'moderation_permission'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})
    


@to_json_decorator
@attr.s
class ChatCreateReqBody(object):
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    chat_mode = attr.ib(type=str, default=None, metadata={'json': 'chat_mode'})
    chat_type = attr.ib(type=str, default=None, metadata={'json': 'chat_type'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    join_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'join_message_visibility'})
    leave_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'leave_message_visibility'})
    membership_approval = attr.ib(type=str, default=None, metadata={'json': 'membership_approval'})
    


@attr.s
class ChatCreateResult(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    avatar = attr.ib(type=str, default=None, metadata={'json': 'avatar'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    i18n_names = attr.ib(type=I18nNames, default=None, metadata={'json': 'i18n_names'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    owner_id_type = attr.ib(type=str, default=None, metadata={'json': 'owner_id_type'})
    add_member_permission = attr.ib(type=str, default=None, metadata={'json': 'add_member_permission'})
    share_card_permission = attr.ib(type=str, default=None, metadata={'json': 'share_card_permission'})
    at_all_permission = attr.ib(type=str, default=None, metadata={'json': 'at_all_permission'})
    edit_permission = attr.ib(type=str, default=None, metadata={'json': 'edit_permission'})
    chat_mode = attr.ib(type=str, default=None, metadata={'json': 'chat_mode'})
    chat_type = attr.ib(type=str, default=None, metadata={'json': 'chat_type'})
    chat_tag = attr.ib(type=str, default=None, metadata={'json': 'chat_tag'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    tenant_key = attr.ib(type=str, default=None, metadata={'json': 'tenant_key'})
    join_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'join_message_visibility'})
    leave_message_visibility = attr.ib(type=str, default=None, metadata={'json': 'leave_message_visibility'})
    membership_approval = attr.ib(type=str, default=None, metadata={'json': 'membership_approval'})
    moderation_permission = attr.ib(type=str, default=None, metadata={'json': 'moderation_permission'})
    



@attr.s
class ChatSearchResult(object):
    items = attr.ib(type=List[ListChat], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})



@attr.s
class MessageGetResult(object):
    items = attr.ib(type=List[Message], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class ChatMembersCreateReqBody(object):
    id_list = attr.ib(type=List[str], default=None, metadata={'json': 'id_list'})


@attr.s
class ChatMembersCreateResult(object):
    invalid_id_list = attr.ib(type=List[str], default=None, metadata={'json': 'invalid_id_list'})


@to_json_decorator
@attr.s
class ChatMembersDeleteReqBody(object):
    id_list = attr.ib(type=List[str], default=None, metadata={'json': 'id_list'})


@attr.s
class ChatMembersDeleteResult(object):
    invalid_id_list = attr.ib(type=List[str], default=None, metadata={'json': 'invalid_id_list'})



@attr.s
class ChatMembersGetResult(object):
    items = attr.ib(type=List[ListMember], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    member_total = attr.ib(type=int, default=None, metadata={'json': 'member_total'})





@attr.s
class ChatAnnouncementGetResult(object):
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})
    revision = attr.ib(type=str, default=None, metadata={'json': 'revision'})
    create_time = attr.ib(type=str, default=None, metadata={'json': 'create_time'})
    update_time = attr.ib(type=str, default=None, metadata={'json': 'update_time'})
    owner_id_type = attr.ib(type=str, default=None, metadata={'json': 'owner_id_type'})
    owner_id = attr.ib(type=str, default=None, metadata={'json': 'owner_id'})
    modifier_id_type = attr.ib(type=str, default=None, metadata={'json': 'modifier_id_type'})
    modifier_id = attr.ib(type=str, default=None, metadata={'json': 'modifier_id'})



@attr.s
class ChatMembersIsInChatResult(object):
    is_in_chat = attr.ib(type=bool, default=None, metadata={'json': 'is_in_chat'})




@to_json_decorator
@attr.s
class ChatAnnouncementPatchReqBody(object):
    revision = attr.ib(type=str, default=None, metadata={'json': 'revision'})
    requests = attr.ib(type=List[str], default=None, metadata={'json': 'requests'})



@attr.s
class MessageReceiveEventData(object):
    sender = attr.ib(type=EventSender, default=None, metadata={'json': 'sender'})
    message = attr.ib(type=EventMessage, default=None, metadata={'json': 'message'})


@attr.s
class MessageReceiveEvent(BaseEventV2):
    event = attr.ib(type=MessageReceiveEventData, default=None)



@attr.s
class ChatUpdatedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})
    after_change = attr.ib(type=ChatChange, default=None, metadata={'json': 'after_change'})
    before_change = attr.ib(type=ChatChange, default=None, metadata={'json': 'before_change'})
    moderator_list = attr.ib(type=ModeratorList, default=None, metadata={'json': 'moderator_list'})


@attr.s
class ChatUpdatedEvent(BaseEventV2):
    event = attr.ib(type=ChatUpdatedEventData, default=None)



@attr.s
class ChatDisbandedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})


@attr.s
class ChatDisbandedEvent(BaseEventV2):
    event = attr.ib(type=ChatDisbandedEventData, default=None)



@attr.s
class ChatMemberUserAddedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})
    users = attr.ib(type=List[ChatMemberUser], default=None, metadata={'json': 'users'})


@attr.s
class ChatMemberUserAddedEvent(BaseEventV2):
    event = attr.ib(type=ChatMemberUserAddedEventData, default=None)



@attr.s
class ChatMemberBotAddedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})


@attr.s
class ChatMemberBotAddedEvent(BaseEventV2):
    event = attr.ib(type=ChatMemberBotAddedEventData, default=None)



@attr.s
class ChatMemberBotDeletedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})


@attr.s
class ChatMemberBotDeletedEvent(BaseEventV2):
    event = attr.ib(type=ChatMemberBotDeletedEventData, default=None)



@attr.s
class ChatMemberUserWithdrawnEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})
    users = attr.ib(type=List[ChatMemberUser], default=None, metadata={'json': 'users'})


@attr.s
class ChatMemberUserWithdrawnEvent(BaseEventV2):
    event = attr.ib(type=ChatMemberUserWithdrawnEventData, default=None)



@attr.s
class ChatMemberUserDeletedEventData(object):
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    external = attr.ib(type=bool, default=None, metadata={'json': 'external'})
    operator_tenant_key = attr.ib(type=str, default=None, metadata={'json': 'operator_tenant_key'})
    users = attr.ib(type=List[ChatMemberUser], default=None, metadata={'json': 'users'})


@attr.s
class ChatMemberUserDeletedEvent(BaseEventV2):
    event = attr.ib(type=ChatMemberUserDeletedEventData, default=None)



@attr.s
class MessageAtMessageReadEventData(object):
    reader = attr.ib(type=EventMessageReader, default=None, metadata={'json': 'reader'})
    message_id_list = attr.ib(type=List[str], default=None, metadata={'json': 'message_id_list'})


@attr.s
class MessageAtMessageReadEvent(BaseEventV2):
    event = attr.ib(type=MessageAtMessageReadEventData, default=None)



@attr.s
class MessageMessageReadEventData(object):
    reader = attr.ib(type=EventMessageReader, default=None, metadata={'json': 'reader'})
    message_id_list = attr.ib(type=List[str], default=None, metadata={'json': 'message_id_list'})


@attr.s
class MessageMessageReadEvent(BaseEventV2):
    event = attr.ib(type=MessageMessageReadEventData, default=None)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Public(object):
    pass



@to_json_decorator
@attr.s
class PublicGetReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})


@attr.s
class PublicGetResult(object):
    security_entity = attr.ib(type=str, default=None, metadata={'json': 'security_entity'})
    comment_entity = attr.ib(type=str, default=None, metadata={'json': 'comment_entity'})
    share_entity = attr.ib(type=str, default=None, metadata={'json': 'share_entity'})
    link_share_entity = attr.ib(type=str, default=None, metadata={'json': 'link_share_entity'})
    external_access = attr.ib(type=bool, default=None, metadata={'json': 'external_access'})
    invite_external = attr.ib(type=bool, default=None, metadata={'json': 'invite_external'})
    permission_verision = attr.ib(type=str, default=None, metadata={'json': 'permission_verision'})


@to_json_decorator
@attr.s
class PublicUpdateReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    security_entity = attr.ib(type=str, default=None, metadata={'json': 'security_entity'})
    comment_entity = attr.ib(type=str, default=None, metadata={'json': 'comment_entity'})
    share_entity = attr.ib(type=str, default=None, metadata={'json': 'share_entity'})
    link_share_entity = attr.ib(type=str, default=None, metadata={'json': 'link_share_entity'})
    external_access = attr.ib(type=bool, default=None, metadata={'json': 'external_access'})
    invite_external = attr.ib(type=bool, default=None, metadata={'json': 'invite_external'})



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_no_data_field, set_user_access_token, \
    set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.bots = BotService(self)


class BotService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> BotGetReqCall

        request_opts = [set_no_data_field()]  # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return BotGetReqCall(self, request_opts=request_opts)


class BotGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (BotService, List[Any]) -> None

        self.service = service

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[BotGetResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('bot/v3/info', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=BotGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class FaceVerify(object):
    pass




@attr.s
class FaceVerifyCropFaceImageResult(object):
    face_image = attr.ib(type=str, default=None, metadata={'json': 'face_image'})



@attr.s
class FaceVerifyQueryAuthResultResult(object):
    auth_state = attr.ib(type=int, default=None, metadata={'json': 'auth_state'})
    auth_timpstamp = attr.ib(type=int, default=None, metadata={'json': 'auth_timpstamp'})



@attr.s
class FaceVerifyUploadFaceImageResult(object):
    face_uid = attr.ib(type=str, default=None, metadata={'json': 'face_uid'})


# -*- coding: UTF-8 -*-

class APIError(Exception):
    def __init__(self, message=''):  # type: (str) -> None
        super(APIError, self).__init__(message)
        self.message = message


# Errors

ERR_ACCESS_TOKEN_TYPE_INVALID = APIError(message='access token type is invalid')
ERR_TENANT_KEY_IS_EMPTY = APIError(message='tenant key is empty')
ERR_USER_ACCESS_TOKEN_KEY_IS_EMPTY = APIError(message='user access token is empty')
ERR_APP_TICKET_IS_EMPTY = APIError(message='app ticket is empty')



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.messages = MessageService(self)
        self.chats = ChatService(self)
        self.chat_member_users = ChatMemberUserService(self)
        self.chat_member_bots = ChatMemberBotService(self)
        self.chat_announcements = ChatAnnouncementService(self)
        self.chat_memberss = ChatMembersService(self)
        self.files = FileService(self)
        self.images = ImageService(self)
        self.message_resources = MessageResourceService(self)
        



class MessageService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MessageListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MessageListReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (MessagePatchReqBody, str, str, int) -> MessagePatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MessagePatchReqCall(self, body, request_opts=request_opts)

    def reply(self, body, tenant_key=None, timeout=None):
        # type: (MessageReplyReqBody, str, int) -> MessageReplyReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MessageReplyReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (MessageCreateReqBody, str, int) -> MessageCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MessageCreateReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> MessageDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return MessageDeleteReqCall(self, request_opts=request_opts)

    def read_users(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MessageReadUsersReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MessageReadUsersReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MessageGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MessageGetReqCall(self, request_opts=request_opts)


class ChatService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def update(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (ChatUpdateReqBody, str, str, int) -> ChatUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatUpdateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatListReqCall(self, request_opts=request_opts)

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatDeleteReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatGetReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (ChatCreateReqBody, str, int) -> ChatCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return ChatCreateReqCall(self, body, request_opts=request_opts)

    def search(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatSearchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatSearchReqCall(self, request_opts=request_opts)


class ChatMemberUserService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service


class ChatMemberBotService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service


class ChatAnnouncementService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatAnnouncementGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatAnnouncementGetReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (ChatAnnouncementPatchReqBody, str, str, int) -> ChatAnnouncementPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatAnnouncementPatchReqCall(self, body, request_opts=request_opts)


class ChatMembersService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (ChatMembersCreateReqBody, str, str, int) -> ChatMembersCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatMembersCreateReqCall(self, body, request_opts=request_opts)

    def delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (ChatMembersDeleteReqBody, str, str, int) -> ChatMembersDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatMembersDeleteReqCall(self, body, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatMembersGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatMembersGetReqCall(self, request_opts=request_opts)

    def is_in_chat(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatMembersIsInChatReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatMembersIsInChatReqCall(self, request_opts=request_opts)

    def me_join(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> ChatMembersMeJoinReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return ChatMembersMeJoinReqCall(self, request_opts=request_opts)


class FileService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, tenant_key=None, timeout=None):
        # type: (str, int) -> FileCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return FileCreateReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, response_stream=None, timeout=None):
        # type: (str, Union[None, IO], int) -> FileGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return FileGetReqCall(self, request_opts=request_opts)


class ImageService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, tenant_key=None, timeout=None):
        # type: (str, int) -> ImageCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return ImageCreateReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, response_stream=None, timeout=None):
        # type: (str, Union[None, IO], int) -> ImageGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return ImageGetReqCall(self, request_opts=request_opts)


class MessageResourceService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, response_stream=None, timeout=None):
        # type: (str, Union[None, IO], int) -> MessageResourceGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return MessageResourceGetReqCall(self, request_opts=request_opts)



class MessageListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MessageService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_container_id_type(self, container_id_type):
        # type: (str) -> MessageListReqCall
        self.query_params['container_id_type'] = container_id_type
        return self

    def set_container_id(self, container_id):
        # type: (str) -> MessageListReqCall
        self.query_params['container_id'] = container_id
        return self

    def set_start_time(self, start_time):
        # type: (str) -> MessageListReqCall
        self.query_params['start_time'] = start_time
        return self

    def set_end_time(self, end_time):
        # type: (str) -> MessageListReqCall
        self.query_params['end_time'] = end_time
        return self

    def set_page_token(self, page_token):
        # type: (str) -> MessageListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> MessageListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[MessageListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/messages', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MessageListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessagePatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MessageService, MessagePatchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessagePatchReqCall
        self.path_params['message_id'] = message_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/messages/:message_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageReplyReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MessageService, MessageReplyReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessageReplyReqCall
        self.path_params['message_id'] = message_id
        return self

    def do(self):
        # type: () -> Response[Message]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/messages/:message_id/reply', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=Message, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MessageService, MessageCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_receive_id_type(self, receive_id_type):
        # type: (str) -> MessageCreateReqCall
        self.query_params['receive_id_type'] = receive_id_type
        return self

    def do(self):
        # type: () -> Response[Message]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/messages', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=Message, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MessageService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessageDeleteReqCall
        self.path_params['message_id'] = message_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/messages/:message_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageReadUsersReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MessageService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessageReadUsersReqCall
        self.path_params['message_id'] = message_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MessageReadUsersReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_size(self, page_size):
        # type: (int) -> MessageReadUsersReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> MessageReadUsersReqCall
        self.query_params['page_token'] = page_token
        return self

    def do(self):
        # type: () -> Response[MessageReadUsersResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/messages/:message_id/read_users', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MessageReadUsersResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (ChatService, ChatUpdateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatUpdateReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatUpdateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id', 'PUT', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileCreateReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_type(self, file_type):
        # type: (str) -> FileCreateReqCall
        self.body.add_param('file_type', file_type)
        return self

    def set_file_name(self, file_name):
        # type: (str) -> FileCreateReqCall
        self.body.add_param('file_name', file_name)
        return self

    def set_duration(self, duration):
        # type: (int) -> FileCreateReqCall
        self.body.add_param('duration', duration)
        return self

    def set_file(self, file):
        # type: (IO[Any]) -> FileCreateReqCall
        self.body.add_file('file', FormDataFile(file))
        return self

    def do(self):
        # type: () -> Response[FileCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('im/v1/files', 'POST', [ACCESS_TOKEN_TYPE_TENANT], self.body, output_class=FileCreateResult , request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FileGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (FileService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_file_key(self, file_key):
        # type: (str) -> FileGetReqCall
        self.path_params['file_key'] = file_key
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('im/v1/files/:file_key', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> ChatListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> ChatListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[ChatListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ImageCreateReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ImageService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_image_type(self, image_type):
        # type: (str) -> ImageCreateReqCall
        self.body.add_param('image_type', image_type)
        return self

    def set_image(self, image):
        # type: (IO[Any]) -> ImageCreateReqCall
        self.body.add_file('image', FormDataFile(image))
        return self

    def do(self):
        # type: () -> Response[ImageCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('im/v1/images', 'POST', [ACCESS_TOKEN_TYPE_TENANT], self.body, output_class=ImageCreateResult , request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatDeleteReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/chats/:chat_id', 'DELETE', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ImageGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ImageService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_image_key(self, image_key):
        # type: (str) -> ImageGetReqCall
        self.path_params['image_key'] = image_key
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('im/v1/images/:image_key', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatGetReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[ChatGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (ChatService, ChatCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[ChatCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=ChatCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatSearchReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatSearchReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_query(self, query):
        # type: (str) -> ChatSearchReqCall
        self.query_params['query'] = query
        return self

    def set_page_token(self, page_token):
        # type: (str) -> ChatSearchReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> ChatSearchReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[ChatSearchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/search', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatSearchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MessageService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessageGetReqCall
        self.path_params['message_id'] = message_id
        return self

    def do(self):
        # type: () -> Response[MessageGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/messages/:message_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MessageGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatMembersCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (ChatMembersService, ChatMembersCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatMembersCreateReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_member_id_type(self, member_id_type):
        # type: (str) -> ChatMembersCreateReqCall
        self.query_params['member_id_type'] = member_id_type
        return self

    def do(self):
        # type: () -> Response[ChatMembersCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id/members', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=ChatMembersCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatMembersDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (ChatMembersService, ChatMembersDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatMembersDeleteReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_member_id_type(self, member_id_type):
        # type: (str) -> ChatMembersDeleteReqCall
        self.query_params['member_id_type'] = member_id_type
        return self

    def do(self):
        # type: () -> Response[ChatMembersDeleteResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id/members', 'DELETE', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=ChatMembersDeleteResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatMembersGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatMembersService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatMembersGetReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_member_id_type(self, member_id_type):
        # type: (str) -> ChatMembersGetReqCall
        self.query_params['member_id_type'] = member_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> ChatMembersGetReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> ChatMembersGetReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[ChatMembersGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id/members', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatMembersGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MessageResourceGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MessageResourceService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_message_id(self, message_id):
        # type: (str) -> MessageResourceGetReqCall
        self.path_params['message_id'] = message_id
        return self

    def set_file_key(self, file_key):
        # type: (str) -> MessageResourceGetReqCall
        self.path_params['file_key'] = file_key
        return self

    def set_type(self, type):
        # type: (str) -> MessageResourceGetReqCall
        self.query_params['type'] = type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('im/v1/messages/:message_id/resources/:file_key', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatAnnouncementGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatAnnouncementService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatAnnouncementGetReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> ChatAnnouncementGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[ChatAnnouncementGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('im/v1/chats/:chat_id/announcement', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatAnnouncementGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatMembersIsInChatReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatMembersService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatMembersIsInChatReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def do(self):
        # type: () -> Response[ChatMembersIsInChatResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/chats/:chat_id/members/is_in_chat', 'GET', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=ChatMembersIsInChatResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatMembersMeJoinReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ChatMembersService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatMembersMeJoinReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/chats/:chat_id/members/me_join', 'PATCH', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ChatAnnouncementPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (ChatAnnouncementService, ChatAnnouncementPatchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_chat_id(self, chat_id):
        # type: (str) -> ChatAnnouncementPatchReqCall
        self.path_params['chat_id'] = chat_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('im/v1/chats/:chat_id/announcement', 'PATCH', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-

import logging

LEVEL_DEBUG = 1
LEVEL_INFO = 2
LEVEL_WARN = 3
LEVEL_ERROR = 4


class Logger(object):
    def debug(self, message):  # type: (str) -> None
        pass

    def info(self, message):  # type: (str) -> None
        pass

    def warn(self, message):  # type: (str) -> None
        pass

    def error(self, message):  # type: (str) -> None
        pass


class LoggerProxy(Logger):
    """
    A common logger class, developers can specify behaviors when logs are emitted.
    """

    def __init__(self, log_level, logger):  # type: (int, Logger) -> None
        self.log_level = log_level
        self.logger = logger

    def debug(self, message):  # type: (str) -> None
        if self.log_level <= LEVEL_DEBUG:
            self.logger.debug(message)

    def info(self, message):  # type: (str) -> None
        if self.log_level <= LEVEL_INFO:
            self.logger.info(message)

    def warn(self, message):  # type: (str) -> None
        if self.log_level <= LEVEL_WARN:
            self.logger.warn(message)

    def error(self, message):  # type: (str) -> None
        if self.log_level <= LEVEL_ERROR:
            self.logger.error(message)


class DefaultLogger(Logger):
    """
    This is the default logger which emits logs to python logging APIs
    """

    def __init__(self):
        logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.DEBUG)

    def debug(self, message):  # type: (str) -> None
        logging.debug(message)

    def info(self, message):  # type: (str) -> None
        logging.info(message)

    def warn(self, message):  # type: (str) -> None
        logging.warning(message)

    def error(self, message):  # type: (str) -> None
        logging.error(message)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.mailgroup_members = MailgroupMemberService(self)
        self.mailgroup_permission_members = MailgroupPermissionMemberService(self)
        self.public_mailboxs = PublicMailboxService(self)
        self.public_mailbox_members = PublicMailboxMemberService(self)
        self.mailgroups = MailgroupService(self)
        



class MailgroupMemberService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupMemberListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupMemberListReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupMemberGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupMemberGetReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (MailgroupMember, str, int) -> MailgroupMemberCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupMemberCreateReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupMemberDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupMemberDeleteReqCall(self, request_opts=request_opts)


class MailgroupPermissionMemberService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupPermissionMemberDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupPermissionMemberDeleteReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupPermissionMemberGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupPermissionMemberGetReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupPermissionMemberListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupPermissionMemberListReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (MailgroupPermissionMember, str, int) -> MailgroupPermissionMemberCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupPermissionMemberCreateReqCall(self, body, request_opts=request_opts)


class PublicMailboxService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def update(self, body, tenant_key=None, timeout=None):
        # type: (PublicMailbox, str, int) -> PublicMailboxUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxUpdateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxListReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (PublicMailbox, str, int) -> PublicMailboxCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxCreateReqCall(self, body, request_opts=request_opts)

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxGetReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, timeout=None):
        # type: (PublicMailbox, str, int) -> PublicMailboxPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxPatchReqCall(self, body, request_opts=request_opts)


class PublicMailboxMemberService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxMemberGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxMemberGetReqCall(self, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxMemberDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxMemberDeleteReqCall(self, request_opts=request_opts)

    def clear(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxMemberClearReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxMemberClearReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> PublicMailboxMemberListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxMemberListReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (PublicMailboxMember, str, int) -> PublicMailboxMemberCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return PublicMailboxMemberCreateReqCall(self, body, request_opts=request_opts)


class MailgroupService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupGetReqCall(self, request_opts=request_opts)

    def update(self, body, tenant_key=None, timeout=None):
        # type: (Mailgroup, str, int) -> MailgroupUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupUpdateReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupDeleteReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (Mailgroup, str, int) -> MailgroupCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupCreateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> MailgroupListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupListReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, timeout=None):
        # type: (Mailgroup, str, int) -> MailgroupPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return MailgroupPatchReqCall(self, body, request_opts=request_opts)



class MailgroupPermissionMemberDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupPermissionMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupPermissionMemberDeleteReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_permission_member_id(self, permission_member_id):
        # type: (str) -> MailgroupPermissionMemberDeleteReqCall
        self.path_params['permission_member_id'] = permission_member_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/permission_members/:permission_member_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxMemberGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxMemberGetReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def set_member_id(self, member_id):
        # type: (str) -> PublicMailboxMemberGetReqCall
        self.path_params['member_id'] = member_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> PublicMailboxMemberGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[PublicMailboxMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id/members/:member_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=PublicMailboxMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxMemberDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxMemberDeleteReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def set_member_id(self, member_id):
        # type: (str) -> PublicMailboxMemberDeleteReqCall
        self.path_params['member_id'] = member_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id/members/:member_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupMemberListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupMemberListReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupMemberListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupMemberListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> MailgroupMemberListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> MailgroupMemberListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[MailgroupMemberListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/members', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MailgroupMemberListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupPermissionMemberGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupPermissionMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupPermissionMemberGetReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_permission_member_id(self, permission_member_id):
        # type: (str) -> MailgroupPermissionMemberGetReqCall
        self.path_params['permission_member_id'] = permission_member_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupPermissionMemberGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupPermissionMemberGetReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[MailgroupPermissionMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/permission_members/:permission_member_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MailgroupPermissionMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupPermissionMemberListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupPermissionMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupPermissionMemberListReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupPermissionMemberListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupPermissionMemberListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> MailgroupPermissionMemberListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> MailgroupPermissionMemberListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[MailgroupPermissionMemberListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/permission_members', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MailgroupPermissionMemberListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (PublicMailboxService, PublicMailbox, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxUpdateReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def do(self):
        # type: () -> Response[PublicMailbox]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id', 'PUT', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=PublicMailbox, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxMemberClearReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxMemberClearReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id/members/clear', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxMemberListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxMemberListReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> PublicMailboxMemberListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> PublicMailboxMemberListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> PublicMailboxMemberListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[PublicMailboxMemberListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id/members', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=PublicMailboxMemberListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxMemberCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (PublicMailboxMemberService, PublicMailboxMember, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxMemberCreateReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> PublicMailboxMemberCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[PublicMailboxMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id/members', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=PublicMailboxMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupGetReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def do(self):
        # type: () -> Response[Mailgroup]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=Mailgroup, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MailgroupService, Mailgroup, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupUpdateReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def do(self):
        # type: () -> Response[Mailgroup]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id', 'PUT', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=Mailgroup, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_token(self, page_token):
        # type: (str) -> PublicMailboxListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> PublicMailboxListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[PublicMailboxListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/public_mailboxes', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=PublicMailboxListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupDeleteReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupPermissionMemberCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MailgroupPermissionMemberService, MailgroupPermissionMember, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupPermissionMemberCreateReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupPermissionMemberCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupPermissionMemberCreateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[MailgroupPermissionMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/permission_members', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=MailgroupPermissionMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (PublicMailboxService, PublicMailbox, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[PublicMailbox]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('mail/v1/public_mailboxes', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=PublicMailbox, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupMemberGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupMemberGetReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_member_id(self, member_id):
        # type: (str) -> MailgroupMemberGetReqCall
        self.path_params['member_id'] = member_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupMemberGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupMemberGetReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[MailgroupMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/members/:member_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MailgroupMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (PublicMailboxService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxGetReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def do(self):
        # type: () -> Response[PublicMailbox]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=PublicMailbox, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class PublicMailboxPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (PublicMailboxService, PublicMailbox, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_public_mailbox_id(self, public_mailbox_id):
        # type: (str) -> PublicMailboxPatchReqCall
        self.path_params['public_mailbox_id'] = public_mailbox_id
        return self

    def do(self):
        # type: () -> Response[PublicMailbox]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/public_mailboxes/:public_mailbox_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=PublicMailbox, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MailgroupService, Mailgroup, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[Mailgroup]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('mail/v1/mailgroups', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=Mailgroup, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_token(self, page_token):
        # type: (str) -> MailgroupListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> MailgroupListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[MailgroupListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=MailgroupListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MailgroupService, Mailgroup, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupPatchReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def do(self):
        # type: () -> Response[Mailgroup]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=Mailgroup, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupMemberCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (MailgroupMemberService, MailgroupMember, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupMemberCreateReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> MailgroupMemberCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> MailgroupMemberCreateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[MailgroupMember]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/members', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=MailgroupMember, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class MailgroupMemberDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (MailgroupMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_mailgroup_id(self, mailgroup_id):
        # type: (str) -> MailgroupMemberDeleteReqCall
        self.path_params['mailgroup_id'] = mailgroup_id
        return self

    def set_member_id(self, member_id):
        # type: (str) -> MailgroupMemberDeleteReqCall
        self.path_params['member_id'] = member_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('mail/v1/mailgroups/:mailgroup_id/members/:member_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.attachments = AttachmentService(self)
        self.employees = EmployeeService(self)
        



class AttachmentService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, response_stream=None, timeout=None):
        # type: (str, Union[None, IO], int) -> AttachmentGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return AttachmentGetReqCall(self, request_opts=request_opts)


class EmployeeService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> EmployeeListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return EmployeeListReqCall(self, request_opts=request_opts)



class AttachmentGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (AttachmentService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_token(self, token):
        # type: (str) -> AttachmentGetReqCall
        self.path_params['token'] = token
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('ehr/v1/attachments/:token', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class EmployeeListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (EmployeeService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_view(self, view):
        # type: (str) -> EmployeeListReqCall
        self.query_params['view'] = view
        return self

    def set_status(self, status):
        # type: (List[int]) -> EmployeeListReqCall
        self.query_params['status'] = status
        return self

    def set_type(self, type):
        # type: (List[int]) -> EmployeeListReqCall
        self.query_params['type'] = type
        return self

    def set_start_time(self, start_time):
        # type: (int) -> EmployeeListReqCall
        self.query_params['start_time'] = start_time
        return self

    def set_end_time(self, end_time):
        # type: (int) -> EmployeeListReqCall
        self.query_params['end_time'] = end_time
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> EmployeeListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_user_ids(self, user_ids):
        # type: (str) -> EmployeeListReqCall
        self.query_params['user_ids'] = user_ids
        return self

    def set_page_token(self, page_token):
        # type: (str) -> EmployeeListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> EmployeeListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[EmployeeListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('ehr/v1/employees', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=EmployeeListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-

CONTENT_TYPE = "Content-Type"
CONTENT_TYPE_JSON = "application/json"
DEFAULT_CONTENT_TYPE = CONTENT_TYPE_JSON + "; charset=utf-8"

HTTP_HEADER_KEY = "HTTP-Request"
HTTP_HEADER_KEY_REQUEST_ID = "X-Request-Id"
HTTP_HEADER_KEY_LOG_ID = "X-Log-Id"
HTTP_KEY_STATUS_CODE = "http_status_code"

APP_TYPE_ISV = "isv"
APP_TYPE_INTERNAL = "internal"

DOMAIN_FEISHU = "https://open.feishu.cn"
DOMAIN_LARK_SUITE = "https://open.larksuite.com"

CALLBACK_TYPE_EVENT = "event_callback"
CALLBACK_TYPE_CHALLENGE = "url_verification"

SDK_VERSION = "1.0.0"

OAPI_ROOT_PATH = "open-apis"

APP_ACCESS_TOKEN_INTERNAL_URL_PATH = "auth/v3/app_access_token/internal"
APP_ACCESS_TOKEN_ISV_URL_PATH = "auth/v3/app_access_token"

TENANT_ACCESS_TOKEN_INTERNAL_URL_PATH = "auth/v3/tenant_access_token/internal"
TENANT_ACCESS_TOKEN_ISV_URL_PATH = "auth/v3/tenant_access_token"

APPLY_APP_TICKET_PATH = "auth/v3/app_ticket/resend"

ACCESS_TOKEN_TYPE_NONE = 'none_access_token'
ACCESS_TOKEN_TYPE_APP = 'app_access_token'
ACCESS_TOKEN_TYPE_TENANT = 'tenant_access_token'
ACCESS_TOKEN_TYPE_USER = 'user_access_token'

APP_TICKET_KEY_PREFIX = 'app_ticket'
APP_ACCESS_TOKEN_KEY_PREFIX = 'app_access_token'
TENANT_ACCESS_TOKEN_KEY_PREFIX = "tenant_access_token"
EXPIRE_DELTA = 3 * 60

ERR_CODE_NATIVE = -1
ERR_CODE_OK = 0
ERR_CODE_APP_TICKET_INVALID = 10012
ERR_CODE_ACCESS_TOKEN_INVALID = 99991671
ERR_CODE_APP_ACCESS_TOKEN_INVALID = 99991664
ERR_CODE_TENANT_ACCESS_TOKEN_INVALID = 99991663
ERR_CODE_USER_ACCESS_TOKEN_INVALID = 99991668
ERR_CODE_USER_REFRESH_TOKEN_INVALID = 99991669

RETRY_CODE = [ERR_CODE_ACCESS_TOKEN_INVALID,
              ERR_CODE_APP_ACCESS_TOKEN_INVALID, ERR_CODE_TENANT_ACCESS_TOKEN_INVALID]



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class DeleteRecord(object):
    deleted = attr.ib(type=bool, default=None, metadata={'json': 'deleted'})
    record_id = attr.ib(type=str, default=None, metadata={'json': 'record_id'})


@to_json_decorator
@attr.s
class Person(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    en_name = attr.ib(type=str, default=None, metadata={'json': 'en_name'})
    email = attr.ib(type=str, default=None, metadata={'json': 'email'})


@to_json_decorator
@attr.s
class App(object):
    app_token = attr.ib(type=str, default=None, metadata={'json': 'app_token'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class AppTable(object):
    table_id = attr.ib(type=str, default=None, metadata={'json': 'table_id'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class AppTableField(object):
    field_id = attr.ib(type=str, default=None, metadata={'json': 'field_id'})
    field_name = attr.ib(type=str, default=None, metadata={'json': 'field_name'})
    type = attr.ib(type=int, default=None, metadata={'json': 'type'})
    property = attr.ib(type=Any, default=None, metadata={'json': 'property'})


@to_json_decorator
@attr.s
class AppTableRecord(object):
    record_id = attr.ib(type=str, default=None, metadata={'json': 'record_id'})
    fields = attr.ib(type=Dict[str, Any], default=None, metadata={'json': 'fields'})


@to_json_decorator
@attr.s
class Option(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})



@to_json_decorator
@attr.s
class AppTableRecordBatchDeleteReqBody(object):
    records = attr.ib(type=List[str], default=None, metadata={'json': 'records'})


@attr.s
class AppTableRecordBatchDeleteResult(object):
    records = attr.ib(type=List[DeleteRecord], default=None, metadata={'json': 'records'})


@to_json_decorator
@attr.s
class AppTableRecordBatchCreateReqBody(object):
    records = attr.ib(type=List[AppTableRecord], default=None, metadata={'json': 'records'})


@attr.s
class AppTableRecordBatchCreateResult(object):
    records = attr.ib(type=List[AppTableRecord], default=None, metadata={'json': 'records'})



@attr.s
class AppTableRecordGetResult(object):
    record = attr.ib(type=AppTableRecord, default=None, metadata={'json': 'record'})



@attr.s
class AppTableRecordUpdateResult(object):
    record = attr.ib(type=AppTableRecord, default=None, metadata={'json': 'record'})





@attr.s
class AppTableRecordListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[AppTableRecord], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class AppTableRecordBatchUpdateReqBody(object):
    records = attr.ib(type=List[AppTableRecord], default=None, metadata={'json': 'records'})


@attr.s
class AppTableRecordBatchUpdateResult(object):
    records = attr.ib(type=List[AppTableRecord], default=None, metadata={'json': 'records'})



@attr.s
class AppTableRecordCreateResult(object):
    record = attr.ib(type=AppTableRecord, default=None, metadata={'json': 'record'})



@attr.s
class AppGetResult(object):
    app = attr.ib(type=App, default=None, metadata={'json': 'app'})



@attr.s
class AppTableListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[AppTable], default=None, metadata={'json': 'items'})



@attr.s
class AppTableFieldListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[AppTableField], default=None, metadata={'json': 'items'})


# -*- coding: UTF-8 -*-


from typing import Dict, List, Union

from .oapi_header import OapiHeader


class OapiResponse(object):
    def __init__(self, status_code=200, content_type='', body=b'', header=None):
        # type: (int, str, bytes, Union[OapiHeader, Dict[str, Union[None, str, List[str]]]]) -> None
        self.status_code = status_code
        self.content_type = content_type
        self.body = body

        if header is None:
            header = {}

        if isinstance(header, dict):
            self.header = OapiHeader(data=header)
        else:
            self.header = header



# -*- coding: UTF-8 -*-

import time

from ...config import Config
from ...consts import APP_TYPE_INTERNAL
from ...context import Context
from ..handler.manager import Manager
from ..model.event import BaseEvent, BaseEventData
import attr


@attr.s
class AppTicketEventData(BaseEventData):
    app_ticket = attr.ib(type=str, default='')


@attr.s
class AppTicketEvent(BaseEvent):
    event = attr.ib(type=AppTicketEventData, default=None)


def app_ticket_event_callback(ctx, conf, event):
    # type: (Context, Config, AppTicketEvent) -> None
    key = "%s-%s" % ('app_ticket', conf.app_settings.app_id)
    val = event.event.app_ticket
    expire = int(time.time()) + (12 * 60 * 60)
    try:
        conf.store.set(key, val, expire)
    except Exception as e:
        conf.logger.warn('set %s=%s, expire=%d into storage occurred an error:%s' % (key, val, expire, e.__str__()))


def set_app_ticket_event_handler(conf):
    # type: (Config) -> None
    if conf.app_settings.app_type == APP_TYPE_INTERNAL:
        return
    Manager.set_event_callback(conf, 'app_ticket', app_ticket_event_callback, clazz=AppTicketEvent)



# -*- coding: UTF-8 -*-

import json


class JSONSerializable(object):
    def __str__(self):  # type: () -> str
        d = {}

        for key in self.__dict__.keys():
            if key.startswith('__') or key.endswith('__'):
                continue

            d[key] = self.__getattribute__(key)
        return json.dumps(d)


class ISVTenantAccessTokenReq(JSONSerializable):
    def __init__(self, app_access_token='', tenant_key=''):  # type: (str, str) -> None
        self.app_access_token = app_access_token
        self.tenant_key = tenant_key


class TenantAccessToken(JSONSerializable):
    def __init__(self, tenant_access_token='', expire=''):  # type: (str, int) -> None
        self.expire = expire
        self.tenant_access_token = tenant_access_token


class InternalAccessTokenReq(JSONSerializable):
    def __init__(self, app_id='', app_secret=''):  # type: (str, str) -> None
        self.app_id = app_id
        self.app_secret = app_secret


class ISVAppAccessTokenReq(JSONSerializable):
    def __init__(self, app_id='', app_secret='', app_ticket=''):  # type: (str, str, str) -> None
        self.app_id = app_id
        self.app_secret = app_secret
        self.app_ticket = app_ticket


class AppAccessToken(JSONSerializable):
    def __init__(self, app_access_token='', expire=''):  # type: (str, int) -> None
        self.expire = expire
        self.app_access_token = app_access_token


class ApplyAppTicketReq(JSONSerializable):
    def __init__(self, app_id='', app_secret=''):  # type: (str, str) -> None
        self.app_id = app_id
        self.app_secret = app_secret



# -*- coding: UTF-8 -*-

from typing import Any, Callable

from ..config import Config
from ..context import Context
from ..model.oapi_request import OapiRequest
from ..model.oapi_response import OapiResponse
from .app.ticket import set_app_ticket_event_handler
from .handler.handlers import EventHandlers
from .handler.manager import Manager
from .model.event import HTTPEvent


def set_event_callback(conf, event_type, callback, clazz=dict):
    # type: (Config, str, Callable[[Context, Config, Any], Any], Any) -> None

    Manager.set_event_callback(conf, event_type, callback, clazz=clazz)


def handle_event(conf, req):
    # type: (Config, OapiRequest) -> OapiResponse
    http_event = HTTPEvent(request=req)

    if Manager.get_event_callback(conf, 'app_ticket') is None:
        set_app_ticket_event_handler(conf)
    EventHandlers(conf, http_event).handle()
    return http_event.response



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import Callable

from ....config import Config
from ....context import Context
from ....event.event import set_event_callback

from .model import *


class AppOpenEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, AppOpenEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, AppOpenEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, AppOpenEvent], Any]) -> None
        handler = AppOpenEventHandler(callback)
        set_event_callback(conf, "app_open",
                          handler.handle, clazz=AppOpenEvent)


class AppStatusChangeEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, AppStatusChangeEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, AppStatusChangeEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, AppStatusChangeEvent], Any]) -> None
        handler = AppStatusChangeEventHandler(callback)
        set_event_callback(conf, "app_status_change",
                          handler.handle, clazz=AppStatusChangeEvent)


class AppUninstalledEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, AppUninstalledEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, AppUninstalledEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, AppUninstalledEvent], Any]) -> None
        handler = AppUninstalledEventHandler(callback)
        set_event_callback(conf, "app_uninstalled",
                          handler.handle, clazz=AppUninstalledEvent)


class OrderPaidEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, OrderPaidEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, OrderPaidEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, OrderPaidEvent], Any]) -> None
        handler = OrderPaidEventHandler(callback)
        set_event_callback(conf, "order_paid",
                          handler.handle, clazz=OrderPaidEvent)



# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages

PACKAGE = "larksuiteoapi"
VERSION = __import__(PACKAGE).VERSION

setup(
    name='larksuite-oapi',
    version=VERSION,
    description='larksuite oapi python',
    author='rabbit',
    url='https://github.com/larksuite/oapi-sdk-python',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['pycryptodome>=3.0.0', 'attrs>=19.0.0',
                      'requests>=2.20.0', 'typing;python_version<"3.5"'],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ]
)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.admin_dept_stats = AdminDeptStatService(self)
        self.admin_user_stats = AdminUserStatService(self)
        



class AdminDeptStatService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> AdminDeptStatListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return AdminDeptStatListReqCall(self, request_opts=request_opts)


class AdminUserStatService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> AdminUserStatListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return AdminUserStatListReqCall(self, request_opts=request_opts)



class AdminDeptStatListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (AdminDeptStatService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id_type(self, department_id_type):
        # type: (str) -> AdminDeptStatListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_start_date(self, start_date):
        # type: (str) -> AdminDeptStatListReqCall
        self.query_params['start_date'] = start_date
        return self

    def set_end_date(self, end_date):
        # type: (str) -> AdminDeptStatListReqCall
        self.query_params['end_date'] = end_date
        return self

    def set_department_id(self, department_id):
        # type: (str) -> AdminDeptStatListReqCall
        self.query_params['department_id'] = department_id
        return self

    def set_contains_child_dept(self, contains_child_dept):
        # type: (bool) -> AdminDeptStatListReqCall
        self.query_params['contains_child_dept'] = contains_child_dept
        return self

    def set_page_size(self, page_size):
        # type: (int) -> AdminDeptStatListReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> AdminDeptStatListReqCall
        self.query_params['page_token'] = page_token
        return self

    def do(self):
        # type: () -> Response[AdminDeptStatListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('admin/v1/admin_dept_stats', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=AdminDeptStatListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class AdminUserStatListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (AdminUserStatService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_start_date(self, start_date):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['start_date'] = start_date
        return self

    def set_end_date(self, end_date):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['end_date'] = end_date
        return self

    def set_department_id(self, department_id):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['department_id'] = department_id
        return self

    def set_user_id(self, user_id):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['user_id'] = user_id
        return self

    def set_page_size(self, page_size):
        # type: (int) -> AdminUserStatListReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> AdminUserStatListReqCall
        self.query_params['page_token'] = page_token
        return self

    def do(self):
        # type: () -> Response[AdminUserStatListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('admin/v1/admin_user_stats', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=AdminUserStatListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp

